package fr.velossity.sample.device.impl;

import java.util.Random;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.Measurement;

/**
 * A simple device that just keeps the more recent measurement.
 * @author C. Saint-Marcel
 * 
 */
public class MockDeviceImpl implements Device, Runnable {

	private String identifier;

	private Thread measureGenerator;

	/**
	 * The last simulated measurement.
	 */
	private Measurement lastMeasurement;

	/*
	 * Default constructor.
	 */
	public MockDeviceImpl(String identifier) {
		super();
		this.identifier = identifier;
	}

	public String getIdentifier() {
		return identifier;
	}

	/**
	 * @see Device#getType()
	 */
	public String getType() {
		return "Mock";
	}

	/**
	 * Mock device is always functional.
	 * @see Device#getState()
	 */
	public String getState() {
		return DEVICE_STATE_FUNCTIONAL;
	}

	/**
	 * @see Device#getLastMeasure()
	 */
	public Measurement getLastMeasure() {
		return lastMeasurement;
	}

	/**
	 * Just starts the main thread.
	 */
	public synchronized void activate() {
		if(measureGenerator == null) {
			measureGenerator = new Thread(this);
			measureGenerator.start();
		}
	}

	public synchronized void deactivate() {
		measureGenerator = null;
	}

	/**
	 * @see Runnable#run()
	 * Generates a new measurement every 10 seconds.
	 */
	public void run() {
		Thread mg = measureGenerator;
		while (measureGenerator == mg) {
			lastMeasurement = new Measurement(MockDeviceImpl.this, new Random().nextFloat()* 1000, "Watt", System.currentTimeMillis());
			try {
				synchronized (this) {
					wait(10000);	
				}				
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

		}

	}

}
